%*******************************************************************************
% DIFFARG: run the menu system

function dummy = diffarg( arg1 );
% OPTIONAL argument here is a vector of parameters which is taken
% as input; otherwise input is interactive

global firsttime interact paramvec nparam meshlen meshsep thvec xhvec delfact dispform
global impm calcmode impscale
global eoverr ageom d0 minnam shapenam
global conc concs sourcerate count nruns runtimes runavgs

% UNPUBL+
global showdata datmat
% the latter are not used by any other .m file but are made global so that they
% are stored between invocations of Diffarg in a given session 
% UNPUBL-

interact = (nargin == 0);
if firsttime == [], firsttime = 1; end;
% forces one initialisation of all relevant parameters in a MATLAB session
% which are then inherited with each subsequent run of DIFFARG

if interact,
    paramvec = 0:100;
else
    paramvec = arg1;
end;
nparam = 1;
mode = 1;
while mode ~= 6,
    if firsttime == 1,
        paramvec(nparam) = mode;
        nparam = nparam + 1;    % pretend that user had selected mode explicitly
        if mode == 5, firsttime = 0; end; % end of initialisation
    else
        mode = getparam( 'New temperature history', ...
                         'New grain edge age history', ...
                         'New mineral and/or grain geometry', ...
                         'Options', ...
                         'Start model running', ...
                         'Quit program', ...
                         'Enter choice ' );
    end;

    if mode == 1,
        thvec = gethist( 'T' );
%--------------------------------------------------------------------------

    elseif mode == 2,
        xhvec = gethist( 'X' );
%--------------------------------------------------------------------------

    elseif mode == 3,
	minnams = ['H Muscovite';
	           'R Muscovite';
	           '    Biotite';
	           ' Hornblende' ];

	% Activation energies in J/mol, converted to K for rest of program
	% D0 values in mm^2/s, converted to mm^2/Ma for rest of program
	eovs = [ 219000, 167400, 196700, 268200]/8.314;
	d0s  = [      5,   0.03,    7.7,    2.4]*3.169*10^13;
	%        Muscovite (Hamilton et al. 1989)
	%                Muscovite (Robbins 1972)
	%                        Biotite (Harrison et al. 1985)
	%                                Hornblende (Harrison 1981)

	shnams = [ ' plate   ';
	           ' cylinder';
	           ' sphere  ' ];
	as = [ 8.7, 27, 55 ];

	mi = getparam( minnams(1,:), minnams(2,:), minnams(3,:), ...
	               minnams(4,:), 'Select mineral ' );

	eoverr = eovs( mi );
	d0 = d0s( mi );
	minnam = minnams( mi, : );

	ni = getparam( shnams(1,:), shnams(2,:), shnams(3,:), 'Select grain shape ' );

	ageom = as( ni );
	shapenam = shnams( ni, : );
	delfact = ni-1;

	grsize = getparam( 'Enter grain radius (or slab half-width) in mm ' );
	meshlen = 1;
	while rem( meshlen, 2 ) ~= 0,
	    meshlen = getparam( 'Enter the mesh length (an even integer) ' );
	% it must be even to allow Simpson's rule to be applied
	end;
	meshsep  = grsize/meshlen;     % mm

	% if implicit, calculate a matrix to be used repeatedly
	impm = zeros(meshlen);
	for i = 2:meshlen-1,
	% fill in the main part of this matrix
	    impm(i, i-1) = 1-delfact/(2*(i-1));
	    impm(i, i )  = - 2;
	    impm(i, i+1) = 1+delfact/(2*(i-1));
	end;
	impm( 1, 1 ) = -2*(1+delfact);
	impm( 1, 2 ) = 2*(1+delfact);
	impm( meshlen, meshlen-1 ) = 1-delfact/(2*(meshlen-1));
	impm( meshlen, meshlen ) = -2;
%--------------------------------------------------------------------------

    elseif mode == 4,
        calcmode = getparam( 'Fully explicit', ...
                             'Crank-Nicholson', ...
                             'Enter finite difference scheme ' );
	if calcmode == 2, 
	    impscale = getparam( 'Enter scaling factor for Crank-Nicholson timestep ' );
	end;
% UNPUBL+- .. omit 4th option in published version 
        dispform = getparam( 'No display', ...
                             'Summary figure with four graphs', ...
                             'Summary and individual figures', ...
			     'Summary and individual figures with data display toggle', ...
                             'Enter display required ' ) - 1;
% UNPUBL+
	while dispform == 3,	% always interactive, even when in paramvec mode
	    fname = input( ...
    'Enter name of data file (extension of .csv assumed) or q to quit: ', 's' );
	    if fname == 'q',
		dispform = 2; % ignore data display
		showdata = 0;
	    else
	        fname = [fname '.csv'];
	        fid = fopen( fname );
	        if fid == -1,
		    disp([' Cannot open data file ' fname ', try again']);
		else
		    fclose( fid );
		    datmat = csvread( fname ); % this may crash if file is not formatted
		    s = size(datmat);
		    grbndl = datmat(s(1), 1 );
		    grbndr = datmat(s(1), 2 ); 
	 	    actrad = (grbndr - grbndl)/2; % in microns
		    meshsep = actrad/(1000*meshlen);
		    disp( sprintf('Model radius set to %8.5f mm', meshsep*meshlen) );
	% reset to actual value, though can be overriden
		    showdata = 1;
		    dispform = 2;
		end;
	    end;
	end;
% UNPUBL-
		    
	    
%--------------------------------------------------------------------------

    elseif mode == 5,

	nruns = 1;
	maxruns = 100;
	runtimes = zeros( [1,maxruns] );
	runavgs  = zeros( [1,maxruns] );
	runtimes(1) = 0;

	sourcerate = 1; % the rate of increase of apparent age with time is 1 by defn.
	count = 0;      % integer

	conc = [];
	conc(1:meshlen+1) = zeros([1,meshlen+1]) + hist(0, 'X');
	% i.e. assume that grain crystallised at this time, in equilibrium with
	% the excess argon in the environment

	runavgs(1) = hist(0, 'X');
	concs = zeros( maxruns, meshlen+1 );
	concs( 1, : ) = conc;

	for disptype = 1:dispform,
    	    if disptype == 1,
		figure(1);
		clf;
        	subplot(2,2,1);
    	    else
        	figure(2);
		clf;
    	    end;
	    hold off;
	    
	    plot( (0:meshlen)*meshsep, conc );
	    hold on;
% UNPUBL+
            if showdata,
		s = size(datmat);
		grbndl = datmat(s(1), 1 ); % position of left-hand grain boundary in data
		grbndr = datmat(s(1), 2 ); % position of right-hand grain boundary in data
	 	actrad = (grbndr - grbndl)/2; % in microns
		numdat = s(1) - 1; % last row contains starting position
		for i = 1:numdat,
		    age = datmat(i,1);
		    errbar = datmat(i,2);
		    xpos = datmat(i,3);
		    xerr = datmat(i,4)/1000;
	% note that if user has redefined grain radius, profile will be displayed
	% relative to the model grain boundary not to the model grain centre, also note
	% that the "left hand" is displayed in mirror image form
		    if (grbndl <= xpos) &  (xpos <= (grbndl + actrad)),
			rpos = meshlen*meshsep + (grbndl - xpos)/1000; 
			plot( [rpos rpos], [age-errbar age+errbar], 'c' );
		        plot( [rpos-xerr, rpos+xerr], [age age], 'g' );
		    end;
		    if ((grbndr - actrad) <= xpos) & (xpos <= grbndr),
			rpos = meshlen*meshsep + (xpos - grbndr)/1000; 
			plot( [rpos rpos], [age-errbar age+errbar], 'r' );
		        plot( [rpos-xerr, rpos+xerr], [age age], 'g' );
		    end;
		end;
	    end;
% UNPUBL-
	end;

	tel = 1;
	while tel ~= 0,
	   tel = getparam( ...
	         'Enter time to next display in Ma (or 0 to quit, -1 to print) ');
	    if length(tel) > 1,
	        for ii = 1:length(tel),
	            runmod( tel(ii) );
	        end;
	    else     
	        if tel < 0, print;         end;
	        if tel > 0, runmod( tel ); end;
	    end;
	end;
    end;

    if firsttime == 1, mode = mode + 1; end;
% relevant during startup
    if mode == 6, 
        paramvec = paramvec(1:nparam-1);
    end;
% tidy up the parameter vector which experienced users can then reuse
end;
              
