%****************************************************************************
% gethist( type ): get a piecewise linear history of temperature or
% excess argon; type is 'X' or 'T'; gethist returns an
% n x 2 matrix of points defining the history

function hvec = gethist( type );

if type == 'T',
    hisname = 'temperature'; units = 'deg C';
else
    hisname = 'edge age'; units = 'Ma';
end;
histype = getparam( ['Linear ',hisname,' history'], ...
                    ['Flat-top pulse of ', hisname], ...
                    'Piecewise linear', ...
                    'User defined history', ...
                    ['Input required ', hisname, ' history '] );
if histype < 3,
    inivalue = -1;
    while inivalue < 0,
        inivalue = getparam( ['Input initial ', hisname, ' (', units, ') '] );
        if inivalue < 0, disp( 'Must be > 0' ); end;
    end;
end;
if histype == 1,
    decrate = getparam( ...
        ['Input rate of ', hisname, ' decrease (', units, '/Ma) '] );
    if decrate == 0,
        hvec = [0, inivalue; 1, inivalue];
    elseif decrate > 0,
        hvec = [0, inivalue; inivalue/decrate, 0];
    else
        hvec = [0, inivalue; inivalue + 1000, -1000/decrate];
    end;
elseif histype == 2,
    pvalue = -1;
    while pvalue < 0,
        pvalue = getparam( ['Input pulse ', hisname, ' '] );
        if pvalue < 0, disp( 'Must be > 0' ); end;
    end;
    pertstart = -1;
    while pertstart < 0,
        pertstart = getparam( 'Input pulse start time ' );
        if pertstart < 0, disp( 'Must be > 0' ); end;
    end;
    pertend = -1;
    while pertend <= pertstart,
        pertend   = getparam( 'Input pulse finish time ' );
        if pertend <= pertstart, disp( 'Must be later than pulse start' ); end;
    end;
    hvec = [0,inivalue; pertstart,inivalue; pertstart,pvalue; ...
                 pertend,pvalue; pertend,inivalue];
elseif histype == 3,
    hvec = [];
    nextt = 0;
    pos = 1;
    hvec(1,1) = 0;
    while nextt ~= -1,
        hvec(pos, 2) = -1;
        while hvec(pos,2) < 0,
            hvec(pos, 2) = getparam( ...
        ['Input ', type, ' at time ', num2str(hvec(pos,1)), ' Ma '] );
            if hvec(pos,2) < 0,
                disp( 'Must be > 0' );
            end;
        end;
        badtime = 1;
        while badtime,
            nextt = getparam( 'Input next time (-1 to finish) ' );
            if nextt == -1,
                badtime = 0;
            elseif nextt < hvec(pos,1),
                disp( 'Must be later than previous time point' );
            else
                badtime = 0;
                pos = pos + 1;
                hvec(pos,1) = nextt;
            end;
        end;
    end;
elseif histype == 4,
    hvec = [-1, -1];
% a special escape value to cause user's history function to be called
end;
